/***********************************************************
*                      picoQuadcopter                      *
*                                                          *
*                        calc radig                        *
*             (based on code from Ullrich Radig)           *
***********************************************************/
#include "../config.h"


volatile signed int y_old = 0;
volatile signed int y_sum = 0;
volatile signed int y_e_sum = 0;
volatile signed int y_rc_mittelwert = 0;

volatile signed int x_old = 0;
volatile signed int x_sum = 0;
volatile signed int x_e_sum = 0;
volatile signed int x_rc_mittelwert = 0;

volatile signed int gier_old = 0;



typedef struct pid_data {
    S32 integral;
    S32 mittelwert;
    S32 sum;
    S32 err;
    S32 err_max;
    S32 err_min;
    S32 e_sum;
    S32 e_sum_max;
    S32 e_sum_min;
    S32 old;    
    float kp;
    float ki;
    float kd;
} PID_DATA;

static PID_DATA pid_pitch = {0, 0, 0, 0, 3000, -3000, 0, 3000, -3000, 0, 3,  0.025, 2}; 
static PID_DATA pid_roll; 
static PID_DATA pid_yaw;

S32 pid_ctrl(struct pid_data *pid, S32 gyro, S32 acc)
{
    S32 e, out, error;
    
    error = gyro - pid->mittelwert;
    if (abs(error) > 20) {
        if (gyro > pid->mittelwert) pid->mittelwert += 3;
        if (gyro < pid->mittelwert) pid->mittelwert -= 3;
    } else {
        if (gyro > pid->mittelwert) pid->mittelwert++;
        if (gyro < pid->mittelwert) pid->mittelwert--; 
    }
    pid->sum += gyro;
    pid->err = pid->sum - (acc * 70);
    if(pid->err > pid->err_max) pid->err = pid->err_max;
    if(pid->err < pid->err_min) pid->err = pid->err_min;
    pid->sum -= pid->err * 0.005;
    //----------------------------------------
    //PI - Regler
    //e = w - x (Sollwert = 0)
    e = (gyro / 3) + (pid->sum / 333) - (pid->mittelwert / 5);  
    //Integral bilden und begrenzen (I-Anteil)
    pid->e_sum += e; 
    if (pid->e_sum > pid->e_sum_max) pid->e_sum = pid->e_sum_max;
    if (pid->e_sum < pid->e_sum_min) pid->e_sum = pid->e_sum_min;
    //P-Regler Kp * e + Ki * e_sum (P + I-Anteil)
    out = pid->kp * e + (pid->e_sum * pid->ki);
    //D-Regelpart Kd/ta * e (Zeit ta = 0,01s) (+D Anteil)
    out += (pid->kd * (e - pid->old));
    pid->old = e;
    return(out);
}

/**********************************************************************************
*            calc with MotionTracking & user-command for motor control            *
**********************************************************************************/
void calc(C_VAL *curr, ACT *act)
{
    signed int e, gier = 0;

    
    curr->usr_pitch *= 100;
    curr->usr_roll  *= 100;
    curr->usr_yaw   *= 100;
    curr->usr_z     *= 4;

    
    pid_pitch.integral += curr->gyr_pitch;
    pid_roll.integral  += curr->gyr_roll;
    pid_yaw.integral   += curr->gyr_yaw;

    
    
        //---------------------------------------------------------------------------
	//PD Gyro (GIER)
    
    //e = w - x (Sollwert = 0)
    e = (curr->gyr_yaw * 0.5) - (curr->usr_yaw * 0.4);
    
    //P-Regelpart Kp * e
    gier = 4 * e;
    
    //D-Regelpart Kd/ta * e (Zeit ta = 0,01s)
    gier += 10 * (e - gier_old);
    gier_old = e;
        

    act->pitch = pid_ctrl(&pid_pitch, curr->gyr_pitch, curr->acc_x);
    act->roll  = pid_ctrl(&pid_roll,  curr->gyr_roll, curr->acc_y);
    act->yaw   = -gier; // pid_ctrl(&pid_yaw,   curr->gyr_yaw, 0);

/*
        //---------------------------------------------------------------------------
	//Gyro Signal integrieren und an AAC abgleichen
    rc_error = curr->usr_pitch - y_rc_mittelwert;
    
    if (abs(rc_error) > 20)
    {
        if (curr->usr_pitch > y_rc_mittelwert) y_rc_mittelwert += 3;
        if (curr->usr_pitch < y_rc_mittelwert) y_rc_mittelwert -= 3;
    }
    else
    {
        if (curr->usr_pitch > y_rc_mittelwert) y_rc_mittelwert++;
        if (curr->usr_pitch < y_rc_mittelwert) y_rc_mittelwert--; 
    }
 
    y_sum += curr->gyr_pitch;
    
    y_gyro_error = y_sum - (curr->acc_y * 70);
    
    if(y_gyro_error > 3000) y_gyro_error = 3000;
    if(y_gyro_error < -3000) y_gyro_error =-3000;
    
    y_sum -= y_gyro_error * 0.005;
 
    //----------------------------------------
    //PI - Regler
   
    //e = w - x (Sollwert = 0)
    w = (curr->gyr_pitch * 0.3) + (y_sum * 0.003);
      
    e = w - (y_rc_mittelwert*0.2);

    //Integral bilden und begrenzen (I-Anteil)
    y_e_sum += e; 
    if (y_e_sum > 3000) y_e_sum = 3000;
    if (y_e_sum < -3000) y_e_sum = -3000;

    //P-Regler Kp * e + Ki + e_sum_nick1 (P + I-Anteil)
    y_out = 3 * e + (y_e_sum * 0.025);
    
    //D-Regelpart Kd/ta * e (Zeit ta = 0,01s) (+D Anteil)
    //y_out += 2 * (e - y_old);
    y_old = e;
    

    //----------------------------------------
    //Werte mischen
  
	motor_value = rc_ch[RC_GAS] + 70 + y_out + gier;	
	if (motor_value < 5) motor_value = 5;
	if (motor_value > 255) motor_value = 255;
	PWM_Motor[0]= motor_value;

	motor_value = rc_ch[RC_GAS] + 70 - y_out + gier;	
	if (motor_value < 5) motor_value = 5;	
	if (motor_value > 255) motor_value = 255;
	PWM_Motor[1]= motor_value;

    
    
    
	//---------------------------------------------------------------------------
	//Gyro Signal integrieren und an AAC abgleichen	
    rc_error = curr->usr_roll - x_rc_mittelwert;
    
    if (abs(rc_error) > 20)
    {
        if (curr->usr_roll > x_rc_mittelwert) x_rc_mittelwert += 3;
        if (curr->usr_roll < x_rc_mittelwert) x_rc_mittelwert -= 3;
    }
    else
    {
        if (curr->usr_roll > x_rc_mittelwert) x_rc_mittelwert++;
        if (curr->usr_roll < x_rc_mittelwert) x_rc_mittelwert--; 
    }
 
    x_sum += curr->gyr_roll;
    
    x_gyro_error = x_sum - (curr->acc_x * 80);
    
    if(x_gyro_error > 4000) x_gyro_error = 4000;
    if(x_gyro_error < -4000) x_gyro_error =-4000;
    
    x_sum = x_gyro_error * 0.005;
    
    //----------------------------------------
    //PI - Regler
     
    //e = w - x (Sollwert = 0)
    w = (curr->gyr_roll * 0.3) + (x_sum * 0.003);
      
    e = w - (x_rc_mittelwert * 0.2);

    //Integral bilden und begrenzen (I-Anteil)
    x_e_sum += e; 
    if (x_e_sum > 3000) x_e_sum = 3000;
    if (x_e_sum < -3000) x_e_sum = -3000;

    //P-Regler Kp * e + Ki + e_sum_nick1 (P + I-Anteil)
    x_out = 3 * e + (x_e_sum*0.025);
    
    //D-Regelpart Kd/ta * e (Zeit ta = 0,01s) (+D Anteil)
    //x_out += 2 * (e - x_old);
    x_old = e;
   

    //----------------------------------------
    //Werte mischen
 
	motor_value = rc_ch[RC_GAS] + 70 - x_out - gier;	
	if (motor_value < 10) motor_value = 10;
	if (motor_value > 255) motor_value = 255;
	PWM_Motor[2]= motor_value;

	motor_value = rc_ch[RC_GAS] + 70 + x_out - gier;	
	if (motor_value < 10) motor_value = 10;	
	if (motor_value > 255) motor_value = 255;
	PWM_Motor[3]= motor_value;

    
    act->pitch = y_out;
    act->roll  = x_out;
    act->yaw   = -gier;
*/    
}

void calc_reset(void)
{
    pid_pitch.integral = 0; pid_pitch.sum  = 0; pid_pitch.e_sum  = 0; pid_pitch.old  = 0;
    pid_roll.integral  = 0; pid_roll.sum   = 0; pid_roll.e_sum   = 0; pid_roll.old   = 0;
    pid_yaw.integral   = 0; pid_yaw.sum    = 0; pid_yaw.e_sum    = 0; pid_yaw.old    = 0;
}