/***********************************************************
*                      picoQuadcopter                      *
*                       user control                       *
*                                                          *
*             Copyright (C) 2013 by Frank Goetze           *
*                    www.embedded-os.de                    *
************************************************************
*                       control.c                          *
*           DelTang-RX31 / SPBT2632C2A.AT2 Code            *
*                                                          *
***********************************************************/
#include "../config.h"

/*
***********************************************************
*  UART receiver user control
***********************************************************
*/
OS_PACKED typedef struct chan {
    U08      r_chan_value_h;
    U08      value_l;
} OS_PACKED_ATTR CHAN;
OS_PACKED typedef struct rx31data {
    U08      csum;
    U08      u_quali;
    CHAN     channel[7];
} OS_PACKED_ATTR RX31DATA;
union {
    U08      uartRxBuffer[64];
    RX31DATA rx31;
} uartRxFrame;

U08     ModeDetected = 0;
U08     BTstartup    = 0;
U08     RX_flag      = 0;
U08     uartCount    = 0;

const U08 localname[]  = {"AT+AB LocalName picoQuadcopter\r\n"};
const U08 enablebond[] = {"AT+AB EnableBond\r\n"};

void UART_Transmit(const U08 *buffer);

void USART2_IRQHandler(void)
{
    if(USART_GetITStatus(USART2, USART_IT_RXNE) == SET) {
        USART_ClearFlag(USART2, USART_IT_RXNE);
        uartRxFrame.uartRxBuffer[uartCount++] = (U08)USART_ReceiveData(USART2);
        if(uartCount > sizeof(uartRxFrame)) uartCount = 0;
        if(ModeDetected != 1) {
            if((uartRxFrame.uartRxBuffer[uartCount - 1] == 0x0A) && (uartRxFrame.uartRxBuffer[uartCount - 2] == 0x0D)) {
                if(BTstartup != 0) {
                    uartCount = 0;
                    RX_flag = 1;
                }
                BTstartup = 1;    
            }
        } else {  
            if(uartCount == 16) {
                uartCount = 0;
                RX_flag = 1;
            }
        }
    }
}

void UART_Receiver_Init(void)
{
    GPIO_InitTypeDef       GPIO_InitStructure;
    USART_InitTypeDef      USART_InitStructure;
    NVIC_InitTypeDef       NVIC_InitStructure;
    U32                    i;

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB, ENABLE);
    _NOP(); _NOP();

    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_0;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IPD;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_2;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    NVIC_InitStructure.NVIC_IRQChannel                   = USART2_IRQChannel;
    NVIC_InitStructure.NVIC_IRQChannelCmd                = ENABLE;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x0F;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority        = 0x0F;
    NVIC_Init(&NVIC_InitStructure);

    USART_DeInit(USART2);
    USART_InitStructure.USART_BaudRate            = 115200;
    USART_InitStructure.USART_WordLength          = USART_WordLength_8b;
    USART_InitStructure.USART_StopBits            = USART_StopBits_1;
    USART_InitStructure.USART_Parity              = USART_Parity_No;
    USART_InitStructure.USART_Mode                = USART_Mode_Rx | USART_Mode_Tx;
    USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
    USART_Init(USART2, &USART_InitStructure);

    USART_ITConfig(USART2, USART_IT_RXNE, ENABLE);
    
    USART_Cmd(USART2, ENABLE);

    if(0 != GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_0)) {
        // IR-receiver init
    } else {
        // detect startup message from BT-modul SPBT2632C2A.AT2
        i = 0; while((RX_flag != 1) && (i++ < 0x00ffffff)); 
        if(RX_flag == 1) {             // wait for "AT-AB -CommandMode-\r\nAT-AB BDAddress 00...\r\n"
            RX_flag = 0;
            UART_Transmit(localname);
            while(RX_flag != 1);       // wait for "AT-AB LocalNameOk\r\n"
            RX_flag = 0;
            UART_Transmit(enablebond);
            while(RX_flag != 1);       // wait for "AT-AB EnableBondOk\r\n"
            RX_flag = 0;
            do {                       // now host have to establish the connection ...
                while(RX_flag != 1);   // loop for "AT-AB ConnectionUp\r\n" + "AT-AB -BypassMode-\r\n"
                RX_flag = 0;
            } while (uartRxFrame.uartRxBuffer[17] != '-');
        } else {
            // switch to DelTang-RX31 mode (2-stopBits)
            uartCount = 0;
            USART_InitStructure.USART_StopBits = USART_StopBits_2;
            USART_Init(USART2, &USART_InitStructure);
        }
    }
    ModeDetected = 1;
}

void UART_Transmit(const U08 *buffer)
{
    U08 i = 0;
    
    do {
        USART_SendData(USART2, buffer[i]);
        while(USART_GetFlagStatus(USART2, USART_FLAG_TC) != SET);
    } while(buffer[i++] != 0x0A);
}

BOOLEAN UART_Receive(CHANNEL *buffer)
{
    U08 sum, i;
	
    if(ModeDetected == 0)
        return FALSE;
    RX_flag = 0;
    sum = 0;
    for (i = 1; i < 16; i++) sum += uartRxFrame.uartRxBuffer[i];
    if(sum == uartRxFrame.uartRxBuffer[0]) {
//        if(uartRxFrame.rx31.u_quali & 0x80) {
            for(i = 0; i < 7; i++) {
                buffer->channel[(uartRxFrame.rx31.channel[i].r_chan_value_h & 0x1C) >> 2] = 
                    ((U16)(uartRxFrame.rx31.channel[i].r_chan_value_h & 0x03) << 8) |
                    uartRxFrame.rx31.channel[i].value_l;
            }
            return TRUE;
//        }
    }
    return FALSE;
}
