/***********************************************************
*                      picoQuadcopter                      *
*                       IMU MPU9150                        *
*                                                          *
*             Copyright (C) 2013 by Frank Goetze           *
*                    www.embedded-os.de                    *
************************************************************
*                          imu.c                           *
*                     IMU MPU9150 Code                     *
*                                                          *
***********************************************************/
#include "../config.h"

/*
***********************************************************
*  MPU9150 MotionTracking
***********************************************************
*/
U08 i2cRxBuffer[32];

void I2C_MPU_ByteWrite(U08 slaveAddr, U08 value, U08 writeAddr)
{
    while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY));                              // While the bus is busy
    I2C_GenerateSTART(I2C1, ENABLE);                                            // Send START condition
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));                 // Test on EV5 and clear it
    I2C_Send7bitAddress(I2C1, slaveAddr, I2C_Direction_Transmitter);            // Send MPU9150 address for write
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));   // Test on EV6 and clear it
    I2C_SendData(I2C1, writeAddr);                                              // Send the MPU9150's internal address to write to
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));            // Test on EV8 and clear it
    I2C_SendData(I2C1, value);                                                  // Send the byte to be written
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));            // Test on EV8 and clear it
    I2C_GenerateSTOP(I2C1, ENABLE);                                             // Send STOP condition
} 

void I2C_MPU_BufferRead(U08 slaveAddr, U08 *pBuffer, U08 readAddr, U16 NumByteToRead)
{
    while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY));                              // While the bus is busy
    I2C_GenerateSTART(I2C1, ENABLE);                                            // Send START condition
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));                 // Test on EV5 and clear it
    I2C_Send7bitAddress(I2C1, slaveAddr, I2C_Direction_Transmitter);            // Send MPU9150 address for write
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));   // Test on EV6 and clear it
    I2C_Cmd(I2C1, ENABLE);                                                      // Clear EV6 by setting again the PE bit
    I2C_SendData(I2C1, readAddr);                                               // Send the MPU9150's internal address to write to
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));            // Test on EV8 and clear it
    I2C_GenerateSTART(I2C1, ENABLE);                                            // Send STRAT condition a second time
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));                 // Test on EV5 and clear it
    I2C_Send7bitAddress(I2C1, slaveAddr, I2C_Direction_Receiver);               // Send MPU9150 address for read
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));      // Test on EV6 and clear it
    while(NumByteToRead) {                                                      // While there is data to be read
        if(NumByteToRead == 1) {
            I2C_AcknowledgeConfig(I2C1, DISABLE);                               // Disable Acknowledgement
            I2C_GenerateSTOP(I2C1, ENABLE);                                     // Send STOP Condition
        }
        if(I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED)) {              // Test on EV7 and clear it
            *pBuffer = I2C_ReceiveData(I2C1);                                   // Read a byte from the MPU9150
            pBuffer++;                                                          // Point to the next location where the byte read will be saved
            NumByteToRead--;                                                    // Decrement the read bytes counter
        }
    }
    I2C_AcknowledgeConfig(I2C1, ENABLE);                                        // Enable Acknowledgement to be ready for another reception
}

BOOLEAN I2C_MotionTracking_Init(void)
{
    GPIO_InitTypeDef       GPIO_InitStructure;
    I2C_InitTypeDef        I2C_InitStructure;
    U32                    i;
    U08                    id;

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB, ENABLE);
    _NOP(); _NOP();

    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_6 | GPIO_Pin_7;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_OD;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
     
    I2C_DeInit(I2C1);
    I2C_InitStructure.I2C_Mode                = I2C_Mode_I2C;
    I2C_InitStructure.I2C_Ack                 = I2C_Ack_Enable;
    I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
    I2C_InitStructure.I2C_ClockSpeed          = 100000;
    I2C_InitStructure.I2C_DutyCycle           = I2C_DutyCycle_2;
    I2C_InitStructure.I2C_OwnAddress1         = I2C_OWN_ADDRESS;
    I2C_Cmd(I2C1, ENABLE);
    I2C_Init(I2C1, &I2C_InitStructure);

    for(i = 0; i < 0x00ffffff; i++) _NOP();
    I2C_MPU_ByteWrite(MPU9150_ADD_DEF, 1, MPU9150_REG_PWR_MGMT_1);
    for(i = 0; i < 0x008fffff; i++) _NOP();
    I2C_MPU_BufferRead(MPU9150_ADD_DEF, &id, MPU9150_REG_WHO_AM_I, 1);
    if(id == 0x68) {
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, MPU9150_DEVICE_RESET, MPU9150_REG_PWR_MGMT_1);
        for(i = 0; i < 100000; i++) _NOP();
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, MPU9150_CLK_ZGYRO, MPU9150_REG_PWR_MGMT_1);
        for(i = 0; i < 100000; i++) _NOP();
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, MPU9150_DLPF_CFG_2, MPU9150_REG_CONFIG);
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, MPU9150_LP_WAKE_1_25, MPU9150_REG_PWR_MGMT_2);
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, 1, MPU9150_REG_SMPRT_DIV);
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, GYRO_FS_SEL, MPU9150_REG_GYRO_CONFIG);
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, ACCEL_AFS_SEL, MPU9150_REG_ACCEL_CONFIG);
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, MPU9150_I2C_BYPASS_EN | MPU9150_INT_PIN_RD_CLEAR_EN, MPU9150_REG_INT_PIN_CFG);
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, 1, MPU9150_REG_MOT_THR);
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, 1, MPU9150_REG_MOT_DUR);
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, 100, MPU9150_REG_FF_THR);
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, 1, MPU9150_REG_FF_DUR);
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, MPU9150_MOT_COUNT_1 | MPU9150_FF_COUNT_1, MPU9150_REG_MOT_DETECT_CTRL);
        I2C_MPU_ByteWrite(MPU9150_ADD_DEF, MPU9150_INT_MOT | MPU9150_INT_FF, MPU9150_REG_INT_ENABLE);
        I2C_MPU_ByteWrite(MPU9150_ADD_MAG, 1, MPU9150_REG_CNTL);
        I2C_MPU_BufferRead(MPU9150_ADD_MAG, &id, MPU9150_REG_WIA, 1);
        if(id == 0x48) {
            return TRUE;
        }
    }
    return FALSE;
}

void MPU_getValues(MOTION *buffer)
{
    // handle MPU-INT event
    I2C_MPU_BufferRead(MPU9150_ADD_DEF, i2cRxBuffer, MPU9150_REG_ACCEL_XOUT_H, 14);
    I2C_MPU_BufferRead(MPU9150_ADD_MAG, &i2cRxBuffer[14], MPU9150_REG_HXL, 6);
    // accel x,y,z (from big-endian)
    buffer->accel_x = ((U16)i2cRxBuffer[0] << 8) | i2cRxBuffer[1];
    buffer->accel_y = ((U16)i2cRxBuffer[2] << 8) | i2cRxBuffer[3];
    buffer->accel_z = ((U16)i2cRxBuffer[4] << 8) | i2cRxBuffer[5];
    // gyro x,y,z (from big-endian)
    buffer->gyro_x = ((U16)i2cRxBuffer[8] << 8) | i2cRxBuffer[9];
    buffer->gyro_y = ((U16)i2cRxBuffer[10] << 8) | i2cRxBuffer[11];
    buffer->gyro_z = ((U16)i2cRxBuffer[12] << 8) | i2cRxBuffer[13];
    // mag x,y,z (from little-endian)
    buffer->mag_x = ((U16)i2cRxBuffer[17] << 8) | i2cRxBuffer[16];
    buffer->mag_y = ((U16)i2cRxBuffer[15] << 8) | i2cRxBuffer[14];
    buffer->mag_z = (((U16)i2cRxBuffer[19] << 8) | i2cRxBuffer[18]) & 0x7FFF;
}

