/***********************************************************
*                      picoQuadcopter                      *
*                        motors PWM                        *
*                                                          *
*             Copyright (C) 2013 by Frank Goetze           *
*                    www.embedded-os.de                    *
************************************************************
*                        motors.c                          *
*          normal motors / ESCs BLDC motors Code           *
*                                                          *
***********************************************************/
#include "../config.h"

/***********************************************************
* !! the calculated PWM range (0...1000 or 1000...2000)    *
*    results in a none-linear aero behavior depending      *
*    on the used motors & propellers                    !! *
***********************************************************/

U08 BLDC_detected = 0;
static U16 zero_value = 0;

void PWM_Motors_setPulse(MOTORS *m);

/*
***********************************************************
*  PWM motor 
***********************************************************
*/
void Init_PWM_Motors(void)
{
    GPIO_InitTypeDef         GPIO_InitStructure;
    TIM_TimeBaseInitTypeDef  TIM_TimeBase_InitStructure;
    TIM_OCInitTypeDef        TIM_OC_InitStructure;
    TIM_BDTRInitTypeDef      TIM_BDTR_InitStructure;

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1 | RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB, ENABLE);
 
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_PP;
    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IPD;
    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_1;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
 
    TIM_TimeBase_InitStructure.TIM_ClockDivision     = TIM_CKD_DIV1;
    TIM_TimeBase_InitStructure.TIM_CounterMode       = TIM_CounterMode_Up;
    TIM_TimeBase_InitStructure.TIM_Prescaler         = 72 - 1;        // 1MHz periode base
    if (GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_1) != 0) {
        TIM_TimeBase_InitStructure.TIM_Period        = 1000 - 1;      // 1kHz periode
        zero_value    = 0;                                            // value 0..1000 (0..100%) drives normal motors
        BLDC_detected = 0;
    } else {
        TIM_TimeBase_InitStructure.TIM_Period        = 20000 - 1;     // 50Hz (20ms) periode
        zero_value    = 1000;                                         // value 1000..2000 (1ms..2ms) drives ESCs BLDC
        BLDC_detected = 1;
    }
    TIM_TimeBase_InitStructure.TIM_RepetitionCounter = 0;
    TIM_TimeBaseInit(TIM1, &TIM_TimeBase_InitStructure);
 
    TIM_OC_InitStructure.TIM_OCMode       = TIM_OCMode_PWM1;
    TIM_OC_InitStructure.TIM_OCIdleState  = TIM_OCIdleState_Reset;
    TIM_OC_InitStructure.TIM_OCNIdleState = TIM_OCNIdleState_Set;
    TIM_OC_InitStructure.TIM_OCPolarity   = TIM_OCPolarity_High;
    TIM_OC_InitStructure.TIM_OCNPolarity  = TIM_OCNPolarity_High;
    TIM_OC_InitStructure.TIM_OutputState  = TIM_OutputState_Enable;
    TIM_OC_InitStructure.TIM_OutputNState = TIM_OutputNState_Disable;
    TIM_OC_InitStructure.TIM_Pulse        = zero_value + ((BLDC_detected)? 999 : 0);
    TIM_OC1Init(TIM1, &TIM_OC_InitStructure);
    TIM_OC1PreloadConfig(TIM1, TIM_OCPreload_Enable);
    TIM_OC2Init(TIM1, &TIM_OC_InitStructure);
    TIM_OC2PreloadConfig(TIM1, TIM_OCPreload_Enable);
    TIM_OC3Init(TIM1, &TIM_OC_InitStructure);
    TIM_OC3PreloadConfig(TIM1, TIM_OCPreload_Enable);
    TIM_OC4Init(TIM1, &TIM_OC_InitStructure);
    TIM_OC4PreloadConfig(TIM1, TIM_OCPreload_Enable);

    TIM_BDTR_InitStructure.TIM_OSSRState       = TIM_OSSRState_Enable;
    TIM_BDTR_InitStructure.TIM_OSSIState       = TIM_OSSIState_Enable;
    TIM_BDTR_InitStructure.TIM_LOCKLevel       = TIM_LOCKLevel_OFF;
    TIM_BDTR_InitStructure.TIM_DeadTime        = 10;
    TIM_BDTR_InitStructure.TIM_Break           = TIM_Break_Disable;
    TIM_BDTR_InitStructure.TIM_BreakPolarity   = TIM_BreakPolarity_High;
    TIM_BDTR_InitStructure.TIM_AutomaticOutput = TIM_AutomaticOutput_Disable;
    TIM_BDTRConfig(TIM1, &TIM_BDTR_InitStructure);
    TIM_CCPreloadControl(TIM1, ENABLE);

    TIM_Cmd(TIM1, ENABLE);
    TIM_CtrlPWMOutputs(TIM1, ENABLE);
   
    if(BLDC_detected) {
        MOTORS off = { 0, 0, 0, 0};
        U32    i;
        
        for(i = 0; i < 0x010fffff; i++) _NOP(); 
        PWM_Motors_setPulse(&off);
    }
}

void PWM_Motors_setPulse(MOTORS *m)
{

    TIM1->CCR1 = zero_value + ((m->motor[0] < 1000)? m->motor[0] : 999);
    TIM1->CCR2 = zero_value + ((m->motor[1] < 1000)? m->motor[1] : 999);
    TIM1->CCR3 = zero_value + ((m->motor[2] < 1000)? m->motor[2] : 999);
    TIM1->CCR4 = zero_value + ((m->motor[3] < 1000)? m->motor[3] : 999);
}

void PWM_Motors_test(void)
{
    U16 on_v = ((BLDC_detected)? 100 : 25);
    MOTORS on  = {on_v,on_v,on_v,on_v};
    MOTORS off = { 0, 0, 0, 0};
    U32    i;

    PWM_Motors_setPulse(&on);
    for(i = 0; i < 0x000fffff; i++) _NOP(); 
    PWM_Motors_setPulse(&off);
}

/*
***********************************************************
*  motor mixer for X-mode
***********************************************************
*/
#define RANGE_UINT16(in) (U16)((in<0)?0:((in>999)?999:in))

void MotorMixer(S16 throttle, ACT *act, MOTORS *m)
{
    // neutral range via user-Z
    if(throttle < 30) {
        m->motor[0] = m->motor[1] = m->motor[2] = m->motor[3] = 0;
        calc_reset();
        return;
    }
    // mix actuator values to x-mode motors
    act->pitch /= 2; act->roll /= 2; act->yaw /= 2;
    m->motor[0] = RANGE_UINT16(throttle - act->pitch - act->roll - act->yaw);
    m->motor[1] = RANGE_UINT16(throttle + act->pitch - act->roll + act->yaw);
    m->motor[2] = RANGE_UINT16(throttle - act->pitch + act->roll + act->yaw);
    m->motor[3] = RANGE_UINT16(throttle + act->pitch + act->roll - act->yaw);
}


