#include "calc.h"

typedef struct pid_data {
    S32 integral;
    S32 kp;
    S32 ki;
    S32 err_sum;
    S32 err_sum_max;
    S32 err_sum_min;
    S32 kd;
    S32 err_old;
    S32 out_max;
    S32 out_min;
} PID_DATA;

static PID_DATA pid_pitch = {0,3,2,0,+1024,-1024,1,0,+256,-256}; 
static PID_DATA pid_roll  = {0,3,2,0,+1024,-1024,1,0,+256,-256}; 
static PID_DATA pid_yaw   = {0,3,2,0,+1024,-1024,1,0,+256,-256};

#define LIMIT(val, min, max) (((val) < (min)) ? (min) : (((val) > (max)) ? (max) : (val)))
S32 pid_ctrl(struct pid_data *pid, S32 error)
{
    S32 out = 0;
  
    if (pid->kp != 0)
        out += pid->kp * error;
    if (pid->ki != 0) {
        pid->err_sum += error;
        pid->err_sum = LIMIT(pid->err_sum, pid->err_sum_min, pid->err_sum_max);
        out += (pid->err_sum / pid->ki);
    }
    if (pid->kd != 0) {
        out += pid->kd * (error - pid->err_old);
        pid->err_old = error;
    }
    return LIMIT(out, pid->out_min, pid->out_max);
} 

/**********************************************************************************
*            calc with MotionTracking & user-command for motor control            *
**********************************************************************************/
#define mix_integral   1023
#define mix_acc         300

#define integral_faktor 256
#define gyro_faktor       4

void calc(C_VAL *curr, ACT *act)
{
    S32  pitch, roll, yaw;
    
    curr->usr_pitch *=9;
    curr->usr_roll  *=9;
    curr->usr_yaw   *=9;
    curr->usr_z     *=2;

    pid_pitch.integral += curr->gyr_pitch;
    pid_roll.integral  += curr->gyr_roll;
    pid_yaw.integral   += curr->gyr_yaw;
    
    pid_pitch.integral = ((pid_pitch.integral * mix_integral) + (-curr->acc_y * mix_acc)) / 1024;
    pid_roll.integral  = ((pid_roll.integral  * mix_integral) + ( curr->acc_x * mix_acc)) / 1024;   
    
    pitch = (pid_pitch.integral / integral_faktor) + (curr->gyr_pitch * gyro_faktor) / 16;
    roll  = (pid_roll.integral  / integral_faktor) + (curr->gyr_roll  * gyro_faktor) / 16;
    yaw   = (pid_yaw.integral   / integral_faktor) + (curr->gyr_yaw   * gyro_faktor) / 16; 
    
    act->pitch = pid_ctrl(&pid_pitch,  curr->usr_pitch + pitch);
    act->roll  = pid_ctrl(&pid_roll,    curr->usr_roll + roll );
    act->yaw   = pid_ctrl(&pid_yaw,     curr->usr_yaw  - yaw  );
}

void calc_reset(void)
{
    pid_pitch.integral = 0; pid_pitch.err_sum  = 0; pid_pitch.err_old  = 0;
    pid_roll.integral  = 0; pid_roll.err_sum   = 0; pid_roll.err_old   = 0;
    pid_yaw.integral   = 0; pid_yaw.err_sum    = 0; pid_yaw.err_old    = 0;
}