#include "calc.h"

typedef struct Axis3f {
    float x, y, z;
} AXIS3f;

typedef struct pid_data {
    float angle;
    float integral;
    float kp;
    float ki;
    float err_sum;
    S32   err_sum_max;
    S32   err_sum_min;
    float kd;
    float err_old;
    S32 out_max;
    S32 out_min;
} PID_DATA;

static PID_DATA pid_pitch = {0,0,70.0,0.2,0,+500,-500,3.5,0,+100,-100}; 
static PID_DATA pid_roll  = {0,0,70.0,0.2,0,+500,-500,3.5,0,+100,-100}; 
static PID_DATA pid_yaw   = {0,0,70.0,0.2,0,+500,-500,3.5,0,+100,-100};

#define LIMIT(val, min, max) (S32)(((val) < (min)) ? (min) : (((val) > (max)) ? (max) : (val)))
S32 pid_ctrl(struct pid_data *pid, float error)
{
    float out = 0;
  
    if (pid->kp != 0)
        out += pid->kp * error;
    if (pid->ki != 0) {
        pid->err_sum += error;
        pid->err_sum = LIMIT(pid->err_sum, pid->err_sum_min, pid->err_sum_max);
        out += (pid->err_sum / pid->ki);
    }
    if (pid->kd != 0) {
        out += pid->kd * (error - pid->err_old);
        pid->err_old = error;
    }
    return LIMIT(out, pid->out_min, pid->out_max);
} 

/**********************************************************************************
*            calc with MotionTracking & user-command for motor control            *
**********************************************************************************/
#define IMU_READ_MS    (1.0f / IMU_READ_FREQ)
#define mix_angle      1023
#define mix_acc         300

#define integral_faktor 256
#define gyro_faktor       4

void calc(C_VAL *curr, ACT *act)
{
    float pitch, roll, yaw;

    curr->usr_z *=2;
    
    // angle in 0.1 units
    pid_pitch.angle += ((float)curr->gyr_pitch * IMU_READ_MS * NORM_G * 10) / 32768;
    pid_roll.angle  += ((float)curr->gyr_roll  * IMU_READ_MS * NORM_G * 10) / 32768;
    pid_yaw.angle   += ((float)curr->gyr_yaw   * IMU_READ_MS * NORM_G * 10) / 32768;
/*    
    pid_pitch.integral = (pid_pitch.angle * mix_angle) + (-curr->acc_y * mix_acc);
    pid_roll.integral  = (pid_roll.angle  * mix_angle) + ( curr->acc_x * mix_acc);   
    pid_yaw.integral   = (pid_yaw.angle   * mix_angle);   
    
    pitch = (pid_pitch.integral / integral_faktor) + (curr->gyr_pitch * gyro_faktor);
    roll  = (pid_roll.integral  / integral_faktor) + (curr->gyr_roll  * gyro_faktor);
    yaw   = (pid_yaw.integral   / integral_faktor) + (curr->gyr_yaw   * gyro_faktor); 
*/    
    act->pitch = pid_ctrl(&pid_pitch,  curr->usr_pitch + pid_pitch.angle);
    act->roll  = pid_ctrl(&pid_roll,    curr->usr_roll + pid_roll.angle );
    act->yaw   = pid_ctrl(&pid_yaw,     curr->usr_yaw  - pid_yaw.angle  );
}

void calc_reset(void)
{
    pid_pitch.angle = 0; pid_pitch.integral = 0; pid_pitch.err_sum  = 0; pid_pitch.err_old  = 0;
    pid_roll.angle  = 0; pid_roll.integral  = 0; pid_roll.err_sum   = 0; pid_roll.err_old   = 0;
    pid_yaw.angle   = 0; pid_yaw.integral   = 0; pid_yaw.err_sum    = 0; pid_yaw.err_old    = 0;
}