#include "calc.h"

#define WINDUP_GUARD_GAIN  100.0
#define pgain              50
#define igain              0.0001
#define dgain             -0.045

typedef struct smooth {
    S16  array[16];
	  U08  i;
} SMOOTH;

static S32 pTerm, iTerm, dTerm;

static S32 iPitchState   = 0;
static S32 iRollState    = 0;
static S32 iYawState     = 0;

static S16 last_pitchpos = 0;
static S16 last_rollpos  = 0;
static S16 last_yawpos   = 0;

SMOOTH rollSmooth  = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
SMOOTH pitchSmooth = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
SMOOTH yawSmooth   = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
 
S32 updatePID(S16 targetPosition, S16 currentPosition, S16 *lastPosition, S32 *iState)
{
    S32 error;
    FP windupGaurd;
  
    error = targetPosition - currentPosition;            // determine how badly we are doing
    pTerm = pgain * error;                               // the pTerm is the view from now, the pgain judges how much we care about error we are this instant.
    *iState += error;                                    // iState keeps changing over time; it's overall "performance" over time, or accumulated error
    windupGaurd = WINDUP_GUARD_GAIN / igain;             // to prevent the iTerm getting huge despite lots of error, we use a "windup guard" not necessary, but this makes windup guard values relative to the current iGain
    if (*iState > windupGaurd)       *iState =  windupGaurd;
    else if (*iState < -windupGaurd) *iState = -windupGaurd;
    iTerm = igain * (*iState);
    dTerm = (dgain * (currentPosition - *lastPosition)); // the dTerm, the difference between the current and last reading, indicated the "speed," how quickly the reading is changing. (aka. Differential)
    *lastPosition = currentPosition;
    return  pTerm + iTerm - dTerm;
} 

S16 smooth(S16 rawIn, SMOOTH *smooth) {
    S32 total = 0;
    U08 j;

    smooth->i = (smooth->i + 1) % 16;
    smooth->array[smooth->i] = rawIn;
    for (j = 0; j < 16; j++) total += smooth->array[j];
    return (total  / 16);
} 

/**********************************************************************************
*            calc with MotionTracking & user-command for motor control            *
**********************************************************************************/
void calc(C_VAL *curr, ACT *act)
{
    S16 gyr_p, gyr_r, gyr_y;

    // Removes noise from sensor data 
    gyr_p = smooth(curr->gyr_pitch, &pitchSmooth);
    gyr_r = smooth(curr->gyr_roll,  &rollSmooth);
    gyr_y = smooth(curr->gyr_yaw,   &yawSmooth);
    
    // Update PID with user control & gyro (for X-mode)
    act->pitch = updatePID(gyr_p, -curr->usr_pitch, &last_pitchpos, &iPitchState);
    act->roll  = updatePID(gyr_r, -curr->usr_roll,  &last_rollpos,  &iRollState );
    act->yaw   = updatePID(gyr_y,  curr->usr_yaw,   &last_yawpos,   &iYawState  );    
}


