#include "calc.h"

#include <math.h>

#define timeStep  0.01

#define AccGain    3.9      //8g
#define GyroGain    70     //2000dps
#define PI         3.1415

float giroVar      = 0.1;
float deltaGiroVar = 0.1;
float accelVar     = 5;
float Pxx          = 0.1; // angle variance
float Pvv          = 0.1; // angle change rate variance
float Pxv          = 0.1; // angle and angle change rate covariance
float kx, kv;

int pitchAccel = 0;
int pitchGyro  = 0;
int rollAccel  = 0;
int rollGyro   = 0;

float pitchPrediction = 0; //Output of Kalman filter, final pitch value
float rollPrediction = 0;  //Output of Kalman filter, final roll value
float yawRaw=0; //output of yaw calculator

float xMAGMap;
float yMAGMap;
float zMAGMap;

#define Kp 3   //proportional
#define Ki 10   //Integral
#define Kd -15   //derivative

int ePID[3]  = {0,0,0}; //-error-
int waPID[3] = {0,0,0}; //-the wanted angle for each axis
int caPID[3] = {0,0,0}; //-   current
int pPID[3]  = {0,0,0}; //holds proportional term for each axis
int iPID[3]  = {0,0,0}; //-integral-
int dPID[3]  = {0,0,0}; //-derivitive-
int pePID[3] = {0,0,0}; //-previous error-
int oPID[3]  = {0,0,0}; //final ouput of PID calculations





void YawCalc(C_VAL *curr) // calculate yaw from mag
{
	
	float newACCx, newACCy, pitch, roll, norm, xh, yh, zh;
	
  //YAW!
  //coonvert raw acc to g.
  newACCx = curr->acc_x;
  newACCy = curr->acc_y;

  newACCx = newACCx / pow(2, 15) * 8;
  newACCy = newACCy / pow(2, 15) * 8;

  pitch = asin(newACCx);
  roll = asin(newACCy/cos(pitch));

  //Map the incoming Data from -1 to 1
  xMAGMap = float(map(MAGx, xMAGMin, xMAGMax, -30000, 30000))/30000.0;
  yMAGMap = float(map(MAGy, yMAGMin, yMAGMax, -30000, 30000))/30000.0;
  zMAGMap = float(map(MAGz, zMAGMin, zMAGMax, -30000, 30000))/30000.0;

  //normalize the magnetic vector
  norm = sqrt( sq(xMAGMap) + sq(yMAGMap) + sq(zMAGMap));
  xMAGMap /=norm;
  yMAGMap /=norm;
  zMAGMap /=norm;

  //new calcs:
  xh = xMAGMap * cos(pitch) + zMAGMap * sin(pitch);
  yh = xMAGMap * sin(roll) * sin(pitch) + yMAGMap * cos(roll) - zMAGMap * sin(roll) * cos(pitch);
  zh = -xMAGMap * cos(roll) * sin(pitch) + yMAGMap * sin(roll) + zMAGMap * cos(roll) * cos(pitch);

  yawRaw = 180 * atan2(yh, xh)/PI;
  if (yh >= 0)
  {
    //do nothing, yaw value is ok
  }
  else
  {
    yawRaw += 360;
  }
}

void Kalman(void) //kalman filter for pitch / roll
{
  Pxx += timeStep * (2 * Pxv + timeStep * Pvv);
  Pxv += timeStep * Pvv;
  Pxx += timeStep * giroVar;
  Pvv += timeStep * deltaGiroVar;
  kx = Pxx * (1 / (Pxx + accelVar));
  kv = Pxv * (1 / (Pxx + accelVar));

  pitchPrediction += (pitchAccel - pitchPrediction) * kx;
  rollPrediction += (rollAccel - rollPrediction) * kx;

  Pxx *= (1 - kx);
  Pxv *= (1 - kx);
  Pvv -= kv * Pxv;
}

int PIDCalc(U08 axis)
{
  ePID[axis] = waPID[axis] - caPID[axis];  //finds error between the wanted value and what it currently is.
  pPID[axis] = ePID[axis];                 //"makes a change to the output that is proportional to the current error value"
  iPID[axis] += pError;                    //"if you aren't quite getting to target, keep increasing over a period of time until you get there"
  dPID[axis] = (ePID[axis] - pePID[axis]); //"how fast a change in angle occurs"

  pePID[axis] = ePID[axis]; //puts current error in another value for next loop to use.

  //calculates the final adjusted output based on Ki,Kp,Kd set in the beginning of
  //the program and the resulting values from Proportion, Integral, and Derivative calculations
  // Output = (Kp * Proportion) + (Ki * Integral) + (Kd * Derivative);
  return((Kp * pPID[axis]) + (Ki * iPID[axis]) + (Kd * dPID[axis]));
}

/**********************************************************************************
*            calc with MotionTracking & user-command for motor control            *
**********************************************************************************/
void calc(C_VAL *curr, ACT *act)
{
  	pitchGyro = (pitchGyro + (curr->gyr_pitch / GyroGain)) * timeStep; //gyro pitch in deg
    pitchAccel = (atan2(curr->acc_y / AccGain, curr->acc_z / AccGain) * 180.0) / PI;
    pitchPrediction = pitchPrediction + (curr->gyr_pitch / GyroGain) * timeStep;

    rollGyro = (rollGyro + (curr->gyr_roll / GyroGain)) * timeStep; //gyro roll in deg
    rollAccel = (atan2(curr->acc_x / AccGain, curr->acc_z / AccGain) * 180.0) / PI;
    rollPrediction = rollPrediction - (curr->gyr_roll / GyroGain) * timeStep;

    YawCalc(curr);  //calc yaw with mag!

    Kalman(); //predict pitch, roll
	
// ?? how the values comes into the PID ??

    act->pitch = PIDCalc(0); //ailIn
    act->roll  = PIDCalc(1); //eleIn
    act->yaw   = PIDCalc(2); //rudIn
}
