/***********************************************************
*                      picoQuadcopter                      *
*                       simple main                        *
*                                                          *
*             Copyright (C) 2013 by Frank Goetze           *
*                    www.embedded-os.de                    *
************************************************************
*                  picoQuadcopter_main.c                   *
*                       MAIN Code                          *
*                                                          *
***********************************************************/
#include "config.h"

/******************* global variables ********************/

CHANNEL rx0Buffer = {0,0,0,0,0,0,0,0};
CHANNEL rxBuffer  = {0,0,0,0,0,0,0,0};
MOTIONL mt0Buffer = {0,0,0,0,0,0,0,0,0,0};
MOTION  mtBuffer  = {0,0,0,0,0,0,0,0,0,0};
C_VAL   curr      = {0,0,0,0,0,0,0,0,0,0};
ACT     act       = {0,0,0};
MOTORS  m         = {0,0,0,0};

U08     TIM_flag  = 0;

extern U08 RX_flag;

/*
***********************************************************
*  run LED
***********************************************************
*/
void Init_RunLED(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
    _NOP(); _NOP();
    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_5;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_Write(GPIOB, GPIO_Pin_5);
}

void Toogle_RunLED(void)
{
    if((GPIOB->ODR & GPIO_Pin_5) != 0) GPIOB->ODR &= ~GPIO_Pin_5;
    else                               GPIOB->ODR |=  GPIO_Pin_5;
}



/*
***********************************************************
*  init SysTick
***********************************************************
*/
void Init_SysTick(void)
{
    RCC_ClocksTypeDef  rcc_clocks;
    U32                cnts;
    
    RCC_GetClocksFreq(&rcc_clocks);
    cnts = (U32)rcc_clocks.HCLK_Frequency / IMU_UPDATE_FREQ;
    SysTick_SetReload(cnts);
    SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);
    SysTick_ITConfig(ENABLE);
    SysTick_CounterCmd(SysTick_Counter_Enable);
}

void SysTick_Handler(void)
{
    TIM_flag = 1;
}

/*
***********************************************************
*  MAIN
***********************************************************
*/
#define M_ZEROS 128 
int main(void)
{
    U16     rxLen;
    U08     mt_init;

    NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
    RCC_DeInit();
    RCC_HSEConfig(RCC_HSE_ON);
    if(RCC_WaitForHSEStartUp() == SUCCESS) {
        FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);
        FLASH_SetLatency(FLASH_Latency_2);
        RCC_HCLKConfig(RCC_SYSCLK_Div1); 
        RCC_PCLK1Config(RCC_HCLK_Div2);
        RCC_PCLK2Config(RCC_HCLK_Div1); 
        RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_6);
        RCC_PLLCmd(ENABLE);
        while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET) { }
        RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
        while(RCC_GetSYSCLKSource() != 0x08) { }
    } 

    Init_RunLED();
    UART_Receiver_Init();
    Init_PWM_Motors();
    while(I2C_MotionTracking_Init() == FALSE);
    PWM_Motors_test();
    Init_SysTick();

    mt_init = 0;
    while(1) {
        // wait for flag from UART-RX interrupt (every 22ms)
        while((RX_flag == 0) && (TIM_flag == 0)) _NOP();
        TIM_flag = 0;
        if(RX_flag != 0) UART_Receive(&rxBuffer);
        MPU_getValues(&mtBuffer);
        if((mt_init < M_ZEROS) || (rx0Buffer.channel[0] == 0)) {
            // collect start-values as zero-position of user (only on first RX-event)
            if(rx0Buffer.channel[0] == 0) {
                for(rxLen = 0; rxLen < 7; rxLen++) 
                    rx0Buffer.channel[rxLen] = rxBuffer.channel[rxLen];
            }
            // collect (first M_ZEROS) start-values as zero-position of accel/gyro/mac
            if(mt_init < M_ZEROS) {
                mt0Buffer.accel_x += mtBuffer.accel_x; mt0Buffer.accel_y += mtBuffer.accel_y; mt0Buffer.accel_z += mtBuffer.accel_z;
                mt0Buffer.gyro_x  += mtBuffer.gyro_x;  mt0Buffer.gyro_y  += mtBuffer.gyro_y;  mt0Buffer.gyro_z  += mtBuffer.gyro_z;
                mt0Buffer.mag_x   += mtBuffer.mag_x;   mt0Buffer.mag_y   += mtBuffer.mag_y;   mt0Buffer.mag_z   += mtBuffer.mag_z;
                mt_init++;
            }
            if(mt_init >= M_ZEROS) {
                mt0Buffer.accel_x /= M_ZEROS; mt0Buffer.accel_y /= M_ZEROS; mt0Buffer.accel_z /= M_ZEROS; 
                mt0Buffer.gyro_x  /= M_ZEROS; mt0Buffer.gyro_y  /= M_ZEROS; mt0Buffer.gyro_z  /= M_ZEROS;
                mt0Buffer.mag_x   /= M_ZEROS; mt0Buffer.mag_y   /= M_ZEROS; mt0Buffer.mag_z   /= M_ZEROS;
            }
            if(mt_init & 0x04) Toogle_RunLED();
        } else {
        
            /**********************************************************************************
             *         calc PID with MotionTracking & user-command for motor control          *
             *                                                                                *
             * input:                                                                         *
             *    mtBuffer             =   new values from MPU                                *
             *    mt0Buffer            =   zero-values from MPU at startup                    *
             *    rxBuffer             =   new values from DSM2 or BT receiver                *
             *    rx0Buffer            =   zero-values from DSM2 or BT receiver at startup    *
             *********************************************************************************/

            // sub "null"-values
            curr.acc_x     = mtBuffer.accel_x - (S16)mt0Buffer.accel_x;
            curr.acc_y     = mtBuffer.accel_y - (S16)mt0Buffer.accel_y;
            curr.acc_z     = mtBuffer.accel_z - (S16)mt0Buffer.accel_z;
            curr.gyr_pitch = mtBuffer.gyro_x  - (S16)mt0Buffer.gyro_x;
            curr.gyr_roll  = mtBuffer.gyro_y  - (S16)mt0Buffer.gyro_y;
            curr.gyr_yaw   = mtBuffer.gyro_z  - (S16)mt0Buffer.gyro_z;
            curr.usr_pitch = rxBuffer.channel[CH_pitch]    - rx0Buffer.channel[CH_pitch];
            curr.usr_roll  = rxBuffer.channel[CH_roll]     - rx0Buffer.channel[CH_roll];
            curr.usr_yaw   = rxBuffer.channel[CH_yaw]      - rx0Buffer.channel[CH_yaw];
            curr.usr_z     = rxBuffer.channel[CH_throttle] - rx0Buffer.channel[CH_throttle];
          
            // calc actuators with gyro, accel & mag
            calc(&curr, &act);
            
            // motor mixer for X-mode
            MotorMixer(curr.usr_z, &act, &m);
            
            // write speed to motors
            PWM_Motors_setPulse(&m);

            Toogle_RunLED();
        }
    }
}




